#!/usr/local/bin/perl -w
# Change above line to path to your perl binary

use GD;
use strict;

# ------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
#																					DATA
# ------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
my $unit = "";
my %time_table = ();
my $min_processed_time = 0;
my $max_processed_time = 0;

# ------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
#																					SETTINGS
my $scale = "logarithmic";
my $width = 800;
my $height = 600;
my $padding_x0 = 0.15;
my $padding_x1 = 0.2;
my $padding_y = 0.3;
my $descent = 20;
my $text_offset = -10;
# ------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

ParseInputFile();

# Create a new image
my $im = new GD::Image($width, $height);

# Make the background transparent and interlaced
$im->transparent(Color(255, 255, 255));
$im->interlaced('true');

DrawImage();

# Open a file for writing 
open(PICTURE, ">picture.png") or die("Cannot open file for writing");

# Make sure we are writing to a binary stream
binmode PICTURE;

# Convert the image to PNG and print it to the file PICTURE
print PICTURE $im->png;
close PICTURE;

sub Color {
	my $r = shift;
	my $g = shift;
	my $b = shift;
	return $im->colorAllocate($r, $g, $b);
}

sub ParseInputFile {
	open INPUT, "data.txt" or die "Failed to open input file";
	while (<INPUT>) {
		if ($_ =~ /\s*#/) {next;}
		if ($_ =~ /\[(\d*)\s*:\s*(.*)\]/) {
			$time_table{$1} = $2;
		} elsif ($_ =~ /\s*(.*)\s*:\s*(.*)\s*$/) {
			print "$1 is $2\n";
			if ($1 eq "Size" && $2 =~ /(\d+)x(\d+)/) {
				$width = $1;
				$height = $2;
			} elsif ($1 eq "Scale") {
			$scale = $2;
			}
		}
	}
	close INPUT;
	my $first = 1;
	for (keys %time_table) {
		my $lt = ProcessTime($_);
		if ($first || $lt < $min_processed_time) {$min_processed_time = $lt;}
		if ($first || $lt > $max_processed_time) {$max_processed_time = $lt;}
		$first = 0;
	}
}

sub BottomMost {return $height * (1.0 - $padding_y);}
sub TopMost {return $height - TopMost();}
sub RightMost {return $width * (1.0 - $padding_x1);}
sub LeftMost {return $width * $padding_x0;}
sub TimelineY {return $height / 2;}
sub TimelineWidth {return int(RightMost() - LeftMost() + 0.5);}

sub DrawImage {	
	# draw border
	$im->rectangle(0, 0, $width - 1, $height - 1, Color(0, 0, 0));
	
	# draw line in center
	$im->line(LeftMost(), TimelineY(), RightMost(), TimelineY(), Color(0, 0, 0));
		
	my $indicator_size = ($height / 2) * (1 - $padding_y);
	my $updown = 1;
	foreach (sort {-($a <=> $b)}(keys %time_table)) {
		my $x = Place($_);
		my $indicator = TimelineY() + ($indicator_size * $updown);
		$im->line($x, TimelineY(), $x, $indicator, Color(0, 0, 0));
		my @bounds = GD::Image->stringFT(Color(0, 0, 0), "./verdana.ttf", 12, 0, $x, $indicator, $time_table{$_});
		
		my $rendered_w = $bounds[2] - $bounds[0];
		my $rendered_h = $bounds[3] - $bounds[5];
		
		my $offset_x = $text_offset;
		my $offset_y = -3;
		if ($updown > 0) {
			$offset_y = $rendered_h;
		}
		
		$im->stringFT(Color(0, 0, 0), "./verdana.ttf", 12, 0, $x + $offset_x, $indicator + $offset_y, $time_table{$_});
				
		$updown *= -1;
		$indicator_size -= $descent;
		if ($indicator_size < $descent) {$indicator_size = ($height / 2) * (1 - $padding_y);}
	}
}

sub Place {
	my $time = shift;
	my $norm = (ProcessTime($time) - $min_processed_time) / ($max_processed_time - $min_processed_time);
	return RightMost() - int(TimelineWidth() * $norm);
}

sub ProcessTime {
	my $n = shift;
	if ($scale eq "linear") {return $n;}
	elsif ($scale eq "logarithmic") {return log($n);}
	die "Invalid scale: $scale";
}











